# Generated by Django 2.0.5 on 2025-08-28 00:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_reusable', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='error',
            options={'ordering': ['-last_seen']},
        ),
    ]
